import { zIndexManager } from "./util/zIndexManager.js";

export class Toast {
  static #zIndexManager = new zIndexManager();
  autoClose = null;

  // required
  parents = "";
  html = "";

  // options
  toastClass = "";
  transitionSeconds = 0.3;
  holdSeconds = 3;

  beforeOpen = () => {};
  afterOpen = () => {};
  beforeEnd = () => {};
  afterEnd = () => {};

  constructor(parents, html) {
    if (parents === null) throw "parents must be entered";
    if (html === null) throw "html must be entered";

    this.parents = parents;
    this.html = html;
  }

  async open() {
    // create id
    const id = String(Toast.makeID(32));

    // get z-index
    const zIndex = Toast.getZIndex(this.parents);

    // set toast id
    const toastId = "toast_" + id;

    // add default toast css
    if (document.getElementById("frontleToastCSS") === null) {
      let toastCSSElement = document.createElement("style");
      toastCSSElement.setAttribute("id", "frontleToastCSS");
      toastCSSElement.innerHTML = `
          .frontleToast{
              position: fixed;

              maxWidth: 90%;
              width: auto;
              height: auto;

              top: 90%;
              left: 50%;
              transform: translate(-50%, -50%);

              text-align: center;
              word-break: break-all;
              box-sizing: border-box;
              overflow: hidden;
              padding: 7px 20px 9px 20px;
              border-radius: 500px;
              background: rgba(90, 90, 90, 0.8);
              color: #ffffff;
              opacity: 1;
          }
      `;
      document.head.insertBefore(toastCSSElement, document.head.childNodes[0]);
    }

    // add toast
    let toastElement = document.createElement("div");
    toastElement.setAttribute("id", toastId);
    toastElement.className = `frontleToast ${this.toastClass}`;
    toastElement.innerHTML = this.html;
    toastElement.style.zIndex = String(zIndex);
    toastElement.style.opacity = "0";
    toastElement.style.transition = `opacity ease ${this.transitionSeconds}s 0s`;
    document.querySelector(this.parents).append(toastElement);

    // run lifecycle
    await this.beforeOpen(toastId);

    // set close event
    toastElement.addEventListener("click", () => {
      this.close(toastId);
    });

    // start toast animation
    setTimeout(() => {
      toastElement.style.opacity = "1";
    }, 100);

    // end toast animation
    setTimeout(async () => {
      // close toast
      this.autoClose = setTimeout(() => {
        this.close(toastId);
      }, Number(this.holdSeconds) * 1000);

      // run lifecycle
      await this.afterOpen(toastId);
    }, Number(this.transitionSeconds) * 1000 + 100);
  }

  async close(toastId) {
    // stop auto close
    clearTimeout(this.autoClose);

    // run lifecycle
    await this.beforeEnd(toastId);

    // start toast animation
    let toastElement = document.getElementById(toastId);
    if (toastElement !== null) toastElement.style.opacity = "0";

    // end toast animation
    setTimeout(async () => {
      // remove toast
      if (toastElement !== null) toastElement.remove();

      // run lifecycle
      await this.afterEnd(toastId);
    }, Number(this.transitionSeconds) * 1000);
  }

  static getZIndex(parents) {
    return this.#zIndexManager.getHighestZIndex(parents);
  }

  static makeID(length) {
    let result = "";
    let characters =
      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
    let charactersLength = characters.length;
    for (let i = 0; i < length; i++) {
      result += characters.charAt(Math.floor(Math.random() * charactersLength));
    }
    return result;
  }
}
